package uca.baptistearthur.geocaching.ui.overlay

import android.content.Context
import android.util.Log
import android.view.MotionEvent
import androidx.core.content.ContextCompat
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.osmdroid.bonuspack.routing.RoadManager
import org.osmdroid.util.GeoPoint
import org.osmdroid.views.MapView
import org.osmdroid.views.overlay.Overlay
import org.osmdroid.views.overlay.Polyline
import uca.baptistearthur.geocaching.R


class AddMarkerOverlay(val roadManager: RoadManager) : Overlay() {

    private var locations: MutableSet<PlaceMarker> = mutableSetOf()
    private lateinit var roadOverlay: Polyline
    private var newRoadtripOverlayVisible = false;

    fun addPlaces(geoPoints: Collection<GeoPoint>, mapView: MapView)=
        geoPoints.forEach { addMarkerAtGeopoint(it, mapView)}

    private fun addMarkerAtGeopoint(geoPoint: GeoPoint, mapView: MapView){
        val marker = PlaceMarker(mapView, this)
        marker.position = geoPoint
        if(locations.isNotEmpty()) locations.last().setDefaultIcon()
        locations.add(marker)
        locations.forEach{ it.closeInfoWindow()}
        computeIcons(mapView.context)
        mapView.overlays.add(marker)
        computeRoad(mapView)
        computeNewRoadtripOverlay(mapView);
        mapView.invalidate()
    }

    override fun onDoubleTap(e: MotionEvent?, mapView: MapView?): Boolean {

        val proj = mapView?.projection;
        if(proj!=null){
            val geoPoint = proj.fromPixels(e?.x?.toInt()!!, e.y.toInt() ) as GeoPoint
            addMarkerAtGeopoint(geoPoint, mapView)
        }
        return true;
    }

    fun computeIcons(context: Context) {
        if (locations.isNotEmpty()) {
            val flagIcon = ContextCompat.getDrawable(context, R.drawable.roadtrip_marker)!!
            locations.last().icon = flagIcon
            locations.first().icon = flagIcon
        }
    }

    fun computeRoad(mapView: MapView) {
        mapView.overlays.remove(mapView.overlays.find { it is Polyline})
        if (locations.size > 1) {
            CoroutineScope(Dispatchers.IO).launch {
                val road = roadManager.getRoad(ArrayList(locations.map{it.position}))
                withContext(Dispatchers.Main) {
                    roadOverlay = RoadManager.buildRoadOverlay(road)
                    mapView.overlays.add(roadOverlay)
                }
            }
        }
    }

    fun computeNewRoadtripOverlay(mapView: MapView){
        if (locations.size > 1) {
            if(!newRoadtripOverlayVisible) {
                mapView.overlays.add(NewRoadtripOverlay(locations))
                newRoadtripOverlayVisible = true
            }
        }else{
            Log.d("GeoRoad", "TRY DELETE ROADTRIP OVERLAY" + locations.size)
            Log.d("GeoRoad", ""+mapView.overlays.size)
            mapView.overlays.remove(mapView.overlays.find { it is NewRoadtripOverlay})
            newRoadtripOverlayVisible=false
        }
    }
    fun removeMarker(placeMarker: PlaceMarker) = locations.remove(placeMarker);
    fun getMarkerLabel(placeMarker: PlaceMarker) =
        when (placeMarker) {
            locations.first() -> {
                "Start"
            }
            locations.last() -> {
                "Finish"
            }
            else -> {
                "Step " + locations.indexOf(placeMarker);
            }
        }

}