package uca.baptistearthur.geocaching.ui.overlay

import android.app.AlertDialog
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.text.InputFilter
import android.util.Log
import android.view.MotionEvent
import android.widget.EditText
import android.widget.Toast
import androidx.core.content.ContentProviderCompat.requireContext
import androidx.core.content.ContextCompat
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.ViewModelStoreOwner
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.osmdroid.util.GeoPoint

import org.osmdroid.views.MapView
import org.osmdroid.views.overlay.Overlay
import uca.baptistearthur.geocaching.R
import uca.baptistearthur.geocaching.application.RTApplication
import uca.baptistearthur.geocaching.model.Address
import uca.baptistearthur.geocaching.model.Place
import uca.baptistearthur.geocaching.model.RoadTripEntity
import uca.baptistearthur.geocaching.viewModels.RoadTripViewModel
import uca.baptistearthur.geocaching.viewModels.RoadTripViewModelFactory
import java.util.*


class NewRoadtripOverlay(val points: Collection<PlaceMarker>) : Overlay() {

    private var circleRectF=RectF()

    fun getRoadTripViewModelFromOverlay(overlayContext: Context): RoadTripViewModel {
        val roadTripDao = (overlayContext.applicationContext as RTApplication).db.roadTripDAO()
        val viewModelFactory = RoadTripViewModelFactory(roadTripDao)
        return ViewModelProvider(overlayContext as ViewModelStoreOwner, viewModelFactory).get(RoadTripViewModel::class.java)
    }

    override fun draw(canvas: Canvas, mapView: MapView, shadow: Boolean) {

        val circleSize = 300f
        val circlePadding = 20f
        val circleY = canvas.height - circleSize - circlePadding
        circleRectF= RectF(circlePadding, circleY, circlePadding + circleSize, circleY + circleSize)

        val paint = Paint().apply {
            color = Color.WHITE
            style = Paint.Style.FILL
        }

        canvas.drawCircle(
            circleSize / 2 + circlePadding,
            circleY + circleSize / 2,
            circleSize / 2,
            paint
        )

        val iconSize = 180
        val icon = ContextCompat.getDrawable(mapView.context, R.drawable.check)

        val iconX = (circleSize / 2 - iconSize / 2 + circlePadding).toInt()
        val iconY = (circleY + circleSize / 2 - iconSize / 2).toInt()

        icon?.setBounds(iconX, iconY, iconX + iconSize, iconY + iconSize)
        icon?.draw(canvas)
    }

    override fun onSingleTapConfirmed(e: MotionEvent?, mapView: MapView?) =
        if (e != null && circleRectF.contains(e.x, e.y)) {
            mapView?.let{
                createDialog(it.context)
            }
            Log.d("GeoRoad", "CONFIRM : "+points.size)
            true
        }else{
            false
        }

    private fun createDialog(context: Context){
        val input = EditText(context)
        input.filters = arrayOf<InputFilter>(InputFilter.LengthFilter(50))

        val dialog = AlertDialog.Builder(context)
            .setTitle(R.string.newRoadtripDialog)
            .setView(input)
            .setPositiveButton(R.string.confirm) { _, _ ->
                val userInput = input.text.toString()
                if (userInput.isNotBlank()) {

                    val places: MutableList<Place> = points.map { Place(it.position.latitude, it.position.longitude) }.toMutableList()
                    CoroutineScope(Dispatchers.Main).launch {
                        places.forEach{
                            it.initAddress()
                            Log.d("GeoMap", it.address.displayName)
                        }
                    }
                    val newRoadTrip = RoadTripEntity(
                        id = 0, // auto-generated ID
                        name = input.text.trim().toString(),
                        date = Date(),
                        places = places
                    )
                    getRoadTripViewModelFromOverlay(context).insertRoadTrip(newRoadTrip);


                } else {
                    Toast.makeText(
                        context,
                        R.string.emptyTextError,
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }
            .setNegativeButton(R.string.cancel) { dialog, _ ->
                dialog.cancel()
            }
            .create()
        dialog.show()
    }
}